@@ -59,7 +59,7 @@ from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
 
 DEPLOYMENT_COLUMNS = [
     'id', 'blueprint_id', 'created_at', 'updated_at', 'visibility',
-    'tenant_name', 'created_by', 'site_name'
+    'tenant_name', 'created_by', 'site_name', 'labels'
 ]
 DEPLOYMENT_UPDATE_COLUMNS = [
     'id', 'deployment_id', 'tenant_name', 'state', 'execution_id',
@@ -69,10 +69,17 @@ DEPLOYMENT_UPDATE_PREVIEW_COLUMNS = [
     'deployment_id', 'tenant_name', 'state', 'created_at', 'visibility',
     'old_blueprint_id', 'new_blueprint_id'
 ]
+DEPLOYMENT_MODIFICATION_COLUMNS = [
+    'id', 'workflow_id', 'execution_id', 'status', 'tenant_name',
+    'created_at', 'visibility',
+]
 NON_PREVIEW_COLUMNS = ['id', 'execution_id']
 STEPS_COLUMNS = ['entity_type', 'entity_id', 'action']
 DEPENDENCIES_COLUMNS = ['deployment', 'dependency_type', 'dependent_node',
                         'tenant']
+DEP_GROUP_COLUMNS = [
+    'id', 'deployments', 'description', 'default_blueprint_id'
+]
 TENANT_HELP_MESSAGE = 'The name of the tenant of the deployment'
 DEPLOYMENTS_SUMMARY_FIELDS = (['blueprint_id', 'site_name'] +
                               BASE_SUMMARY_FIELDS)
@@ -83,6 +90,13 @@ MACHINE_READABLE_UPDATE_PREVIEW_COLUMNS = [
     'installed_nodes', 'uninstalled_nodes', 'reinstalled_nodes',
     'explicit_reinstall', 'recursive_dependencies'
 ]
+MACHINE_READABLE_MODIFICATION_COLUMNS = [
+    'ended_at', 'node_instances', 'deployment_id', 'blueprint_id',
+    'modified_nodes', 'resource_availability',
+]
+SCHEDULE_TABLE_COLUMNS = ['id', 'deployment_id', 'workflow_id', 'created_at',
+                          'next_occurrence', 'since', 'until', 'stop_on_fail',
+                          'enabled', 'visibility', 'tenant_name', 'created_by']
 
 
 @cfy.group(name='deployments')
@@ -157,6 +171,8 @@ def _print_single_update(deployment_update_dict,
 
 @cfy.command(name='list', short_help='List deployments [manager only]')
 @cfy.options.blueprint_id()
+@click.option('--group-id', '-g')
+@cfy.options.filter_rules
 @cfy.options.sort_by()
 @cfy.options.descending
 @cfy.options.tenant_name_for_list(
@@ -170,6 +186,8 @@ def _print_single_update(deployment_update_dict,
 @cfy.pass_client()
 @cfy.pass_logger
 def manager_list(blueprint_id,
+                 group_id,
+                 filter,
                  sort_by,
                  descending,
                  all_tenants,
@@ -193,15 +211,31 @@ def manager_list(blueprint_id,
 
     deployments = client.deployments.list(sort=sort_by,
                                           is_descending=descending,
+                                          filter_rules=filter,
                                           _all_tenants=all_tenants,
                                           _search=search,
                                           _offset=pagination_offset,
                                           _size=pagination_size,
+                                          _group_id=group_id,
                                           blueprint_id=blueprint_id)
+    _modify_deployments_labels(deployments)
     total = deployments.metadata.pagination.total
     print_data(DEPLOYMENT_COLUMNS, deployments, 'Deployments:')
-    logger.info('Showing {0} of {1} deployments'.format(len(deployments),
-                                                        total))
+
+    base_str = 'Showing {0} of {1} deployments'.format(len(deployments), total)
+    if filter:
+        filtered = deployments.metadata.get('filtered')
+        if filtered is not None:
+            base_str += ' ({} hidden by filter)'.format(filtered)
+    logger.info(base_str)
+
+
+def _modify_deployments_labels(deployments_list):
+    for dep in deployments_list:
+        dep_labels_list = []
+        for raw_label in dep.get('labels', []):
+            dep_labels_list.append(raw_label.key + ':' + raw_label.value)
+        dep['labels'] = ','.join(dep_labels_list)
 
 
 @cfy.command(name='history', short_help='List deployment updates '
@@ -305,6 +339,8 @@ def manager_get_update(deployment_update_id, logger, client, tenant_name):
 @cfy.options.json_output
 @cfy.options.common_options
 @cfy.options.runtime_only_evaluation
+@cfy.options.auto_correct_types
+@cfy.options.reevaluate_active_statuses()
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
@@ -332,7 +368,9 @@ def manager_update(ctx,
                    blueprint_id,
                    visibility,
                    validate,
-                   runtime_only_evaluation):
+                   runtime_only_evaluation,
+                   auto_correct_types,
+                   reevaluate_active_statuses):
     """Update a specified deployment according to the specified blueprint.
     The blueprint can be supplied as an id of a blueprint that already exists
     in the system (recommended).
@@ -409,7 +447,9 @@ def manager_update(ctx,
             reinstall_list,
             preview,
             not dont_update_plugins,
-            runtime_only_evaluation=runtime_only_evaluation
+            runtime_only_evaluation=runtime_only_evaluation,
+            auto_correct_types=auto_correct_types,
+            reevaluate_active_statuses=reevaluate_active_statuses,
         )
 
     if preview:
@@ -937,3 +977,503 @@ def labels_list_to_set(labels_list):
         labels_set.add((key, value))
 
     return labels_set
+
+
+@deployments.group(name='modifications',
+                   short_help="Handle the deployments' modifications")
+@cfy.options.common_options
+def modifications():
+    if not env.is_initialized():
+        env.raise_uninitialized()
+
+
+@modifications.command(name='list',
+                       short_help="List the deployments' modifications")
+@cfy.argument('deployment-id')
+@cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
+@cfy.options.pagination_offset
+@cfy.options.pagination_size
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def list_modifications(deployment_id,
+                       pagination_offset,
+                       pagination_size,
+                       logger,
+                       client,
+                       tenant_name):
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Listing modifications of the deployment %s...', deployment_id)
+    deployment_modifications = client.deployment_modifications.list(
+        deployment_id,
+        _offset=pagination_offset,
+        _size=pagination_size,
+    )
+    flattened = [dict(dm, **dm.context) if dm.get('context') else dm
+                 for dm in deployment_modifications]
+    total = deployment_modifications.metadata.pagination.total
+    print_data(DEPLOYMENT_MODIFICATION_COLUMNS, flattened,
+               'Deployment modifications:')
+    logger.info('Showing %d of %d deployment modifications',
+                len(deployment_modifications), total)
+
+
+@modifications.command(name='get',
+                       short_help="Retrieve information for a deployment's "
+                                  "modification")
+@cfy.argument('deployment-modification-id')
+@cfy.options.tenant_name(required=False,
+                         resource_name_for_help='deployment modification')
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def get_modification(deployment_modification_id,
+                     logger,
+                     client,
+                     tenant_name):
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Retrieving deployment modification %s...',
+                deployment_modification_id)
+    deployment_modification = client.deployment_modifications.get(
+        deployment_modification_id)
+    _print_deployment_modification(deployment_modification)
+
+
+@modifications.command(name='rollback',
+                       short_help="Rollback a deployment's modification")
+@cfy.argument('deployment-modification-id')
+@cfy.options.tenant_name(required=False,
+                         resource_name_for_help='deployment modification')
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def rollback_modification(deployment_modification_id,
+                          logger,
+                          client,
+                          tenant_name):
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Rolling back a deployment modification %s...',
+                deployment_modification_id)
+    deployment_modification = client.deployment_modifications.rollback(
+        deployment_modification_id)
+    _print_deployment_modification(deployment_modification)
+
+
+def _print_deployment_modification(deployment_modification):
+    def print_node_instance(genre, title, modified_only=False):
+        if genre not in deployment_modification['node_instances'] or \
+                not deployment_modification['node_instances'].get(genre):
+            return
+        print_list(
+            [
+                '{0} ({1})'.format(ni.get('id'), ni.get('node_id'))
+                for ni in deployment_modification['node_instances'].get(genre)
+                if not modified_only or ni.get('modification')
+            ],
+            title
+        )
+
+    columns = DEPLOYMENT_MODIFICATION_COLUMNS
+    if get_global_json_output():
+        columns += MACHINE_READABLE_MODIFICATION_COLUMNS
+    dm = (dict(deployment_modification, **deployment_modification.context)
+          if deployment_modification.context else deployment_modification)
+    print_single(columns, dm, 'Deployment Modification:')
+    if not get_global_json_output():
+        if 'modified_nodes' in dm and dm['modified_nodes']:
+            print_list(dm['modified_nodes'].keys(), 'Modified nodes:')
+        if 'node_instances' in dm and dm['node_instances']:
+            print_node_instance('before_modification',
+                                '\nNode instances before modifications:')
+            print_node_instance('before_rollback',
+                                '\nNode instances before rollback:')
+            print_node_instance('added_and_related',
+                                '\nAdded node instances:',
+                                modified_only=True)
+            print_node_instance('removed_and_related',
+                                '\nRemoved node instances:',
+                                modified_only=True)
+
+
+@deployments.group('groups')
+def groups():
+    """Manage deployment groups"""
+
+
+def _format_group(g):
+    """Format a restclient deployment group for display"""
+    return {
+        'id': g['id'],
+        'description': g['description'],
+        'default_blueprint_id': g['default_blueprint_id'],
+        'deployments': str(len(g['deployment_ids']))
+    }
+
+
+@groups.command('list', short_help='List all deployment groups')
+@cfy.pass_client()
+@cfy.pass_logger
+def groups_list(client, logger):
+    groups = [_format_group(g) for g in client.deployment_groups.list()]
+    print_data(DEP_GROUP_COLUMNS, groups, 'Deployment groups:')
+
+
+@groups.command('create', short_help='Create a new deployment group')
+@click.argument('deployment-group-name')
+@cfy.options.inputs
+@cfy.options.group_default_blueprint
+@cfy.options.group_description
+@cfy.pass_client()
+@cfy.pass_logger
+def groups_create(deployment_group_name, inputs, default_blueprint,
+                  description, client, logger):
+    client.deployment_groups.put(
+        deployment_group_name,
+        default_inputs=inputs,
+        blueprint_id=default_blueprint,
+        description=description
+    )
+    logger.info('Group %s created', deployment_group_name)
+
+
+@groups.command('update', short_help='Update a deployment group')
+@click.argument('deployment-group-name')
+@cfy.options.inputs
+@cfy.options.group_default_blueprint
+@cfy.options.group_description
+@cfy.pass_client()
+@cfy.pass_logger
+def groups_update(deployment_group_name, inputs, default_blueprint,
+                  description, client, logger):
+    client.deployment_groups.put(
+        deployment_group_name,
+        default_inputs=inputs,
+        blueprint_id=default_blueprint,
+        description=description
+    )
+    logger.info('Group %s updated', deployment_group_name)
+
+
+@groups.command('extend', short_help='Add deployments to a group')
+@click.argument('deployment-group-name')
+@cfy.options.group_deployment_id
+@cfy.options.group_count
+@cfy.pass_client()
+@cfy.pass_logger
+def groups_extend(deployment_group_name, deployment_id, count,
+                  client, logger):
+    group = client.deployment_groups.add_deployments(
+        deployment_group_name,
+        count=count,
+        deployment_ids=deployment_id or None
+    )
+    logger.info(
+        'Group %s updated. It now contains %d deployments',
+        deployment_group_name, len(group.deployment_ids)
+    )
+
+
+@groups.command('shrink', short_help='Remove deployments from a group')
+@click.argument('deployment-group-name')
+@cfy.options.group_deployment_id
+@cfy.pass_client()
+@cfy.pass_logger
+def groups_shrink(deployment_group_name, deployment_id, client, logger):
+    group = client.deployment_groups.remove_deployments(
+        deployment_group_name,
+        deployment_id
+    )
+    logger.info(
+        'Unlinked deployments %s. Group %s now has %d deployments',
+        list(deployment_id), deployment_group_name, len(group.deployment_ids)
+    )
+
+
+@cfy.group(name='schedule')
+@cfy.options.common_options
+def schedule():
+    """Handle deployments' execution scheduling [manager only]
+    """
+    pass
+
+
+@schedule.command(name='create',
+                  short_help='Schedule a deployment\'s workflow execution')
+@cfy.argument('name')
+@cfy.options.workflow_id(required=True)
+@cfy.options.deployment_id(required=True)
+@cfy.options.parameters
+@cfy.options.allow_custom_parameters
+@cfy.options.force(help=helptexts.FORCE_CONCURRENT_EXECUTION)
+@cfy.options.dry_run
+@cfy.options.wait_after_fail
+@cfy.options.queue
+@cfy.options.common_options
+@cfy.options.from_datetime(required=True, help="Time to schedule from.")
+@cfy.options.to_datetime(required=False, help="Time to schedule until.")
+@cfy.options.frequency
+@cfy.options.count
+@cfy.options.weekdays
+@cfy.options.rrule
+@cfy.options.slip
+@cfy.options.stop_on_fail
+@cfy.assert_manager_active()
+@cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_create(name,
+                    workflow_id,
+                    deployment_id,
+                    parameters,
+                    allow_custom_parameters,
+                    force,
+                    dry_run,
+                    wait_after_fail,
+                    queue,
+                    from_datetime,
+                    to_datetime,
+                    freq,
+                    count,
+                    weekdays,
+                    rrule,
+                    slip,
+                    stop_on_fail,
+                    tenant_name,
+                    client,
+                    logger):
+    """Schedule the execution of a workflow on a given deployment
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Scheduling the execution of workflow `%s` on deployment '
+                '`%s`. Schedule name: %s', workflow_id, deployment_id, name)
+    client.execution_schedules.create(
+        name,
+        deployment_id,
+        workflow_id,
+        execution_arguments={
+            'allow_custom_parameters': allow_custom_parameters,
+            'force': force,
+            'dry_run': dry_run,
+            'queue': queue,
+            'wait_after_fail': wait_after_fail,
+        },
+        parameters=parameters,
+        since=from_datetime,
+        until=to_datetime,
+        frequency=freq,
+        count=count,
+        weekdays=weekdays,
+        rrule=rrule,
+        slip=slip,
+        stop_on_fail=stop_on_fail)
+
+    logger.info('Deployment schedule created successfully')
+
+
+@schedule.command(name='update',
+                  short_help='Update a deployment schedule')
+@cfy.argument('name')
+@cfy.options.common_options
+@cfy.options.from_datetime(required=False, help="Time to schedule from.")
+@cfy.options.to_datetime(required=False, help="Time to schedule until.")
+@cfy.options.frequency
+@cfy.options.count
+@cfy.options.weekdays
+@cfy.options.rrule
+@cfy.options.slip
+@click.option(
+    '--stop-on-fail/--continue-on-fail',
+    required=False,
+    help=helptexts.SCHEDULE_STOP_ON_FAIL
+)
+@cfy.assert_manager_active()
+@cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_update(name,
+                    from_datetime,
+                    to_datetime,
+                    freq,
+                    count,
+                    weekdays,
+                    rrule,
+                    slip,
+                    stop_on_fail,
+                    tenant_name,
+                    client,
+                    logger):
+    """Update an existing schedule for a workflow execution
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Updating deployment schedule %s...', name)
+    client.execution_schedules.update(
+        name,
+        since=from_datetime,
+        until=to_datetime,
+        frequency=freq,
+        count=count,
+        weekdays=weekdays,
+        rrule=rrule,
+        slip=slip,
+        stop_on_fail=stop_on_fail)
+
+    logger.info('Deployment schedule updated successfully')
+
+
+@schedule.command(name='disable',
+                  short_help='Disable a deployment schedule')
+@cfy.argument('name')
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_disable(name, tenant_name, client, logger):
+    """Disable a schedule for a workflow execution
+    """
+    dep_schedule = client.execution_schedules.get(name)
+    if not dep_schedule.enabled:
+        raise CloudifyCliError(
+            'Deployment schedule {} is already disabled'.format(name))
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Disabling deployment schedule %s...', name)
+    client.execution_schedules.update(name, enabled=False)
+    logger.info('Deployment schedule disabled successfully')
+
+
+@schedule.command(name='enable',
+                  short_help='Enable a disabled deployment schedule')
+@cfy.argument('name')
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_enable(name, tenant_name, client, logger):
+    """Enable a previously-disabled schedule for a workflow execution
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    dep_schedule = client.execution_schedules.get(name)
+    if dep_schedule.enabled:
+        raise CloudifyCliError(
+            'Deployment schedule {} is already enabled'.format(name))
+    logger.info('Enabling deployment schedule %s...', name)
+    client.execution_schedules.update(name, enabled=True)
+    logger.info('Deployment schedule enabled successfully')
+
+
+@schedule.command(name='delete',
+                  short_help='Delete a deployment schedule')
+@cfy.argument('name')
+@cfy.options.common_options
+@cfy.options.tenant_name(required=False,
+                         resource_name_for_help='deployment schedule')
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_delete(name, logger, client, tenant_name):
+    """Delete a schedule for a workflow execution"""
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Deleting deployment schedule %s...', name)
+    client.execution_schedules.delete(name)
+    logger.info('Deployment schedule deleted successfully')
+
+
+@schedule.command(name='list',
+                  short_help='List deployment schedules')
+@cfy.options.sort_by()
+@cfy.options.descending
+@cfy.options.tenant_name_for_list(
+    required=False, resource_name_for_help='deployment schedule')
+@cfy.options.all_tenants
+@cfy.options.search
+@cfy.options.pagination_offset
+@cfy.options.pagination_size
+@cfy.options.common_options
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_list(sort_by,
+                  descending,
+                  tenant_name,
+                  all_tenants,
+                  search,
+                  pagination_offset,
+                  pagination_size,
+                  logger,
+                  client):
+    """List all deployment schedules on the manager
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Listing deployment schedules...')
+    schedules = client.execution_schedules.list(sort=sort_by,
+                                                is_descending=descending,
+                                                _all_tenants=all_tenants,
+                                                _search=search,
+                                                _offset=pagination_offset,
+                                                _size=pagination_size)
+
+    print_data(SCHEDULE_TABLE_COLUMNS, schedules, 'Deployment schedules:')
+    total = schedules.metadata.pagination.total
+    logger.info('Showing %s of %s deployment schedules', len(schedules), total)
+
+
+@schedule.command(name='get',
+                  short_help='Retrieve deployment schedule information')
+@cfy.argument('name')
+@click.option(
+    '--preview',
+    required=False,
+    type=int,
+    help="Preview N next dates for the workflow execution to run."
+)
+@cfy.options.common_options
+@cfy.options.tenant_name(required=False,
+                         resource_name_for_help='deployment schedule')
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def schedule_get(name, preview, logger, client, tenant_name):
+    """Retrieve information for a specific deployment schedule
+
+    `NAME` is the name of the deployment schedule to get information on.
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    logger.info('Retrieving execution schedule %s', name)
+    dep_schedule = client.execution_schedules.get(name)
+
+    columns = SCHEDULE_TABLE_COLUMNS
+    extra_columns = ['rule', 'execution_arguments', 'all_next_occurrences']
+    additional_data = {k.replace('_', ' '): v for k, v in dep_schedule.items()
+                       if k not in SCHEDULE_TABLE_COLUMNS + extra_columns}
+    if get_global_json_output():
+        columns += additional_data.keys() + extra_columns
+    print_single(columns, dep_schedule, 'Execution schedule:', max_width=50)
+
+    if not get_global_json_output():
+        print_details(dep_schedule['rule'], 'Scheduling rule:')
+        print_details(dep_schedule['execution_arguments'],
+                      'Execution arguments:')
+        print_details(additional_data, 'Additional data:')
+
+        if preview:
+            if not dep_schedule.enabled:
+                raise CloudifyCliError(
+                    'Deployment schedule {} is disabled, no upcoming '
+                    'occurrences'.format(name))
+            next_occurrences = dep_schedule['all_next_occurrences']
+
+            computed_msg = 'Computed {} upcoming ' \
+                           'occurrences.'.format(len(next_occurrences))
+            listing_msg = ''
+            if len(next_occurrences) > preview:
+                listing_msg = 'Listing first {}:'.format(preview)
+            elif len(next_occurrences) > 0:
+                listing_msg = 'Listing:'
+            logger.info('%s %s', computed_msg, listing_msg)
+            for i, date in enumerate(next_occurrences):
+                if i == preview:
+                    break
+                logger.info('  {:<5d} {}'.format(i + 1, date))
