@@ -448,6 +448,7 @@ def set_visibility(plugin_id, visibility, logger, client):
                             'the blueprint [manager only]')
 @cfy.argument('blueprint-id', required=False)
 @cfy.options.all_blueprints
+@cfy.options.except_blueprints
 @cfy.options.plugin_names
 @cfy.options.plugins_to_latest
 @cfy.options.plugins_all_to_latest
@@ -461,8 +462,12 @@ def set_visibility(plugin_id, visibility, logger, client):
 @cfy.pass_logger
 @cfy.pass_client()
 @cfy.options.force(help=helptexts.FORCE_PLUGINS_UPDATE)
+@cfy.options.auto_correct_types
+@cfy.options.reevaluate_active_statuses(help=helptexts.
+                                        REEVALUATE_ACTIVE_STATUSES_PLUGINS)
 def update(blueprint_id,
            all_blueprints,
+           except_blueprints,
            plugin_names,
            to_latest,
            all_to_latest,
@@ -473,7 +478,9 @@ def update(blueprint_id,
            logger,
            client,
            tenant_name,
-           force):
+           force,
+           auto_correct_types,
+           reevaluate_active_statuses):
     """Update the plugins of all the deployments of the given blueprint
     or any blueprint in case `--all` flag was used instead of providing
     a BLUEPRINT_ID.  This will update the deployments one by one until
@@ -497,6 +504,10 @@ def update(blueprint_id,
         raise CloudifyValidationError(
             'ERROR: Invalid command syntax. Either provide '
             'a BLUEPRINT_ID or use --all flag.')
+    if except_blueprints and not all_blueprints:
+        raise CloudifyValidationError(
+            'ERROR: Invalid command syntax. Cannot list blueprints '
+            'exceptions unless used with --all flag.')
     all_to_minor = bool(all_to_minor)
     if all_to_latest is None:
         all_to_latest = not all_to_minor
@@ -521,7 +532,8 @@ def update(blueprint_id,
         _update_a_blueprint(blueprint_id, plugin_names,
                             to_latest, all_to_latest, to_minor, all_to_minor,
                             include_logs, json_output, logger,
-                            client, force)
+                            client, force, auto_correct_types,
+                            reevaluate_active_statuses)
     elif all_blueprints:
         update_results = {'successful': [], 'failed': []}
         pagination_offset = 0
@@ -531,12 +543,15 @@ def update(blueprint_id,
                 _offset=pagination_offset,
             )
             for blueprint in blueprints:
+                if blueprint.id in except_blueprints:
+                    continue
                 try:
                     _update_a_blueprint(blueprint.id, plugin_names,
                                         to_latest, all_to_latest,
                                         to_minor, all_to_minor,
                                         include_logs, json_output, logger,
-                                        client, force)
+                                        client, force, auto_correct_types,
+                                        reevaluate_active_statuses)
                     update_results['successful'].append(blueprint.id)
                 except CloudifyClientError as ex:
                     update_results['failed'].append(blueprint.id)
@@ -566,13 +581,17 @@ def _update_a_blueprint(blueprint_id,
                         json_output,
                         logger,
                         client,
-                        force):
+                        force,
+                        auto_correct_types,
+                        reevaluate_active_statuses):
     logger.info('Updating the plugins of the deployments of the blueprint '
                 '{}'.format(blueprint_id))
     plugins_update = client.plugins_update.update_plugins(
         blueprint_id, force=force, plugin_names=plugin_names,
         to_latest=to_latest, all_to_latest=all_to_latest,
-        to_minor=to_minor, all_to_minor=all_to_minor
+        to_minor=to_minor, all_to_minor=all_to_minor,
+        auto_correct_types=auto_correct_types,
+        reevaluate_active_statuses=reevaluate_active_statuses,
     )
     events_logger = get_events_logger(json_output)
     execution = execution_events_fetcher.wait_for_execution(
