@@ -16,9 +16,10 @@
 
 
 import os
-import sys
 import copy
 import json
+import uuid
+import click
 import logging
 import logging.config
 
@@ -37,14 +38,13 @@ HIGH_VERBOSE = 3
 MEDIUM_VERBOSE = 2
 LOW_VERBOSE = 1
 NO_VERBOSE = 0
+QUIET = -1
 
 verbosity_level = NO_VERBOSE
-
+json_output = False
 
 _lgr = None
 
-_all_loggers = set()
-
 
 LOGGER = {
     "version": 1,
@@ -60,16 +60,24 @@ LOGGER = {
         "file": {
             "class": "logging.handlers.RotatingFileHandler",
             "formatter": "file",
-            "maxBytes": "5000000",
-            "backupCount": "20"
+            "maxBytes": 5000000,
+            "backupCount": 20
         },
         "console": {
             "class": "logging.StreamHandler",
             "stream": "ext://sys.stdout",
             "formatter": "console"
         }
+    },
+    "loggers": {
+        "cloudify.cli.main": {
+            "handlers": ["console", "file"],
+            "level": "INFO"
+        }
     }
 }
+# logger that goes only to the file, for use when logging table data
+logfile_logger = logging.getLogger('logfile')
 
 
 def get_logger():
@@ -78,20 +86,19 @@ def get_logger():
     return _lgr
 
 
-def all_loggers():
-    return _all_loggers
-
-
 def configure_loggers():
     # first off, configure defaults
     # to enable the use of the logger
     # even before the init was executed.
-    _configure_defaults()
+    logger_config = copy.deepcopy(LOGGER)
+    _configure_defaults(logger_config)
 
     if env.is_initialized():
         # init was already called
         # use the configuration file.
-        _configure_from_file()
+        _configure_from_file(logger_config)
+    _set_loggers_verbosity(logger_config)
+    logging.config.dictConfig(logger_config)
 
     global _lgr
     _lgr = logging.getLogger('cloudify.cli.main')
@@ -105,59 +112,53 @@ def configure_loggers():
         colorama.init(autoreset=True)
 
 
-def _configure_defaults():
+def _set_loggers_verbosity(logger_config):
+    if get_global_json_output():
+        logger_config['loggers']['cloudify.cli.main']['level'] = 'ERROR'
+    for logger in logger_config['loggers'].values():
+        if verbosity_level >= HIGH_VERBOSE:
+            logger['level'] = logging.DEBUG
+        elif verbosity_level == LOW_VERBOSE:
+            logger['level'] = logging.INFO
+        elif verbosity_level <= QUIET:
+            logger['level'] = logging.ERROR
 
-    # add handlers to the main logger
-    logger_dict = copy.deepcopy(LOGGER)
-    logger_dict['loggers'] = {
-        'cloudify.cli.main': {
-            'handlers': list(logger_dict['handlers'].keys())
+
+def _configure_defaults(logger_config):
+    if get_global_json_output():
+        logger_config['loggers']['logfile'] = {
+            "level": "DEBUG",
+            "propagate": False,
+            "handlers": ["file"]
         }
-    }
-    logger_dict['handlers']['file']['filename'] = DEFAULT_LOG_FILE
+        logger_config['handlers']['console']['stream'] = 'ext://sys.stderr'
+
+    logger_config['handlers']['file']['filename'] = DEFAULT_LOG_FILE
     logfile_dir = os.path.dirname(DEFAULT_LOG_FILE)
     if not os.path.exists(logfile_dir):
         os.makedirs(logfile_dir)
 
-    logging.config.dictConfig(logger_dict)
-    logging.getLogger('cloudify.cli.main').setLevel(logging.INFO)
-    _all_loggers.add('cloudify.cli.main')
-
-
-def _configure_from_file():
 
+def _configure_from_file(loggers_config):
     config = CloudifyConfig()
-    logging_config = config.logging
-    loggers_config = logging_config.loggers
-    logfile = logging_config.filename
 
     # set filename on file handler
     logger_dict = copy.deepcopy(LOGGER)
-    logger_dict['handlers']['file']['filename'] = logfile
-    logfile_dir = os.path.dirname(logfile)
+    loggers_config['handlers']['file']['filename'] = config.logging.filename
+    logfile_dir = os.path.dirname(config.logging.filename)
     if not os.path.exists(logfile_dir):
         os.makedirs(logfile_dir)
 
-    # add handlers to every logger
-    # specified in the file
-    loggers = {}
-    for logger_name in loggers_config:
-        loggers[logger_name] = {
-            'handlers': list(logger_dict['handlers'].keys())
+    # add handlers to every logger specified in the file
+    for logger_name, logging_level in config.logging.loggers.items():
+        loggers_config['loggers'][logger_name] = {
+            'handlers': list(logger_dict['handlers'].keys()),
+            'level': logging_level.upper()
         }
-    logger_dict['loggers'] = loggers
-
-    # set level for each logger
-    for logger_name, logging_level in loggers_config.iteritems():
-        log = logging.getLogger(logger_name)
-        level = logging._levelNames[logging_level.upper()]
-        log.setLevel(level)
-        _all_loggers.add(logger_name)
-
-    logging.config.dictConfig(logger_dict)
 
 
 def get_events_logger(json_output):
+    json_output = json_output or get_global_json_output()
 
     def json_events_logger(events):
         """The json events logger prints events as consumable JSON formatted
@@ -166,11 +167,8 @@ def get_events_logger(json_output):
         :param events: The events to print.
         :return:
         """
-        # TODO: Why we're writing directly to stdout here
-        # but use the logger when the --json-output flag isn't passed.
         for event in events:
-            sys.stdout.write('{}\n'.format(json.dumps(event)))
-            sys.stdout.flush()
+            click.echo(json.dumps(event))
 
     def text_events_logger(events):
         """The default events logger prints events as short messages.
@@ -181,7 +179,7 @@ def get_events_logger(json_output):
         for event in events:
             output = logs.create_event_message_prefix(event)
             if output:
-                _lgr.info(output)
+                click.echo(output)
 
     return json_events_logger if json_output else text_events_logger
 
@@ -192,12 +190,30 @@ def set_global_verbosity_level(verbose):
     global verbosity_level
     verbosity_level = verbose
     logs.EVENT_VERBOSITY_LEVEL = verbosity_level
-    if verbosity_level >= HIGH_VERBOSE:
-        for logger_name in all_loggers():
-            logging.getLogger(logger_name).setLevel(logging.DEBUG)
 
 
 def get_global_verbosity():
     """Return the globally set verbosity
     """
     return verbosity_level
+
+
+def set_global_json_output(enabled=False):
+    global json_output
+    json_output = enabled
+
+
+def get_global_json_output():
+    return json_output
+
+
+def output(line):
+    logfile_logger.info(line)
+    click.echo(line)
+
+
+class CloudifyJSONEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, uuid.UUID):
+            return obj.hex
+        return super(CloudifyJSONEncoder, self).default(obj)
